from io import BytesIO
import pathlib
from zipfile import ZipFile
from urllib.request import urlopen


_SVG_DIR: pathlib.Path = pathlib.Path(__file__).parent / "svg"


def download(
    zip_url="https://github.com/hfg-gmuend/openmoji/releases/latest/download/openmoji-svg-color.zip",
):
    """Download the emojis from openmoji."""
    print("[pygame_emojis] Download the zip file from ", zip_url)
    resp = urlopen(zip_url)
    zipfile = ZipFile(BytesIO(resp.read()))

    print("[pygame_emojis] Extract the zip file to ", _SVG_DIR)
    zipfile.extractall(_SVG_DIR)


if __name__ == "__main__":
    # Dowload the raw data
    download()



"""Emoji support for the game."""
import logging
from pathlib import Path
import emoji
import cairosvg
import io

import pygame

from pygame_emojis.emojis_data.download import _SVG_DIR

logger = logging.getLogger("pygame_emojis")


class EmojiNotFound(Exception):
    ...


def find_code(emoji_: str) -> list[str]:
    """Find the unicode values of the emoji.

    Return a list with all the unicodes.
    """
    # Convert the emoji to a byte str of the emoji
    logger.debug(f"{emoji_}")
    emoji_ = emoji.emojize(emoji_)
    logger.debug(f"{bytes(emoji_, 'utf-8')=}")
    # Convert the byte str to the hexadecimal code

    return [f"{ord(e):X}" for e in emoji_]


def find_svg(emoji_: str) -> list[Path]:
    """Find the svg file tha can be used.

    It uses the follwing method:

    * Find the unicode values of the emoji
    * Tries to find files corresponding to the given unicode values
    * If a file or more are found, return the files
    * Otherwise try to find a file that matches less unicode values
    * repeat until there are no values that were matched
    """

    try:
        code_list = find_code(emoji_)
    except Exception as e:
        raise EmojiNotFound(emoji_) from e
    logger.debug(f"{code_list=}")

    while code_list:

        code = "-".join(code_list)
        possible_files = [f for f in _SVG_DIR.rglob(f"{code}*.svg")]
        if possible_files:
            return possible_files

        # Check with less complex code name
        code_list.pop()

    # Return an empty list if no file was found
    return []


def load_emoji(
    emoji_: str, size: tuple[int, int] | int = None
) -> pygame.Surface:
    """Load a surface corresponding to the emoji."""

    possible_files = find_svg(emoji_)
    logger.debug(f"{possible_files=}")
    if possible_files:
        return load_svg(possible_files[-1], size)
    else:
        raise FileNotFoundError(f"No file available for emoji {emoji_}")


def load_svg(filename, size: tuple[int, int] | int = None) -> pygame.Surface:
    """Load a svg image with cairosvg."""
    kwargs = {}
    if size:
        kwargs["parent_width"] = size if isinstance(size, int) else size[0]
        kwargs["parent_height"] = size if isinstance(size, int) else size[1]

    new_bites = cairosvg.svg2png(url=str(filename), **kwargs)
    byte_io = io.BytesIO(new_bites)
    return pygame.image.load(byte_io)


if __name__ == "__main__":
    # This will list all the files availabe for all the emojis known by emoji package
    print(_SVG_DIR)

    for e, v in emoji.EMOJI_DATA.items():
        print(e, v)

        try:
            print(e, find_code(e))
            print("Files: ", find_svg(e))
        except:
            print("Could not translate")
        print(f_name := "{}*.svg".format(e))
        print("Files v2: ", [f for f in _SVG_DIR.rglob(f_name)])



import logging
import pygame
from random import choice
from pygame_emojis import load_emoji, logger

logging.basicConfig()

# Set logging to debug so we see what happens


logger.setLevel(logging.DEBUG)


if __name__ == "__main__":

    pygame.init()
    size = (640, 640)
    screen = pygame.display.set_mode(size)
    clock = pygame.time.Clock()
    done = False

    pygame.font.init()

    emojis_list = [":red_heart:", "👍", "🫶", "🏳️‍🌈", "😍", "🗡️"]

    def change_emoji():
        screen.fill((0, 0, 0))

        surf = load_emoji(choice(emojis_list), size)
        screen.blit(surf, (0, 0))
        pygame.display.update()

    change_emoji()

    # print(dir(text_image))

    while not done:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                done = True
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                done = True
            elif event.type == pygame.KEYDOWN:

                change_emoji()

        clock.tick(20)



"""This was un attempt to make emojis from fonts.

But I never managed to make it work.
"""
import pygame


def available_emojis_fonts() -> list[str]:
    fonts = pygame.sysfont.get_fonts()
    emojis = [font for font in fonts if "emoji" in font]
    return emojis


def default_emoji_font() -> str:
    emoji_fonts = available_emojis_fonts()
    if emoji_fonts:
        return emoji_fonts[0]
    else:
        raise ValueError("No Font could be find in the system.")


if __name__ == "__main__":
    # Code found online and a bit modified
    import pygame, emoji
    import pygame.freetype

    pygame.init()
    screen = pygame.display.set_mode((640, 480))
    clock = pygame.time.Clock()
    done = False
    print(available_emojis_fonts())

    pygame.font.init()
    font = pygame.font.SysFont(default_emoji_font(), 50)
    # font = pygame.font.SysFont("segoe-ui-symbol.ttf", 72)
    font_color = (255, 255, 255, 255)
    font_background = (0, 0, 0, 0)

    text_string = emoji.emojize(":smile:")
    # text_string = u'ujiHelloÁpQ|, World 👍😖'
    # text_string = u'ujiHelloÁpQ|\U0001f44d, World 😖'
    # print(emoji.emojize('Python is :thumbs_up_sign:'))
    # text = font.render("♛", True, font_color, (0, 0))
    # text = font.render(text_string, True, font_color, (0, 0, 0, 0))
    # text = font.render(text_string, True, "black")
    text = font.render("A", True, "black")

    image_size = list(text[0].get_size())
    text_image = pygame.Surface(image_size)
    text_image.fill(font_background)
    pygame.display.flip()

    text_image.blit(text[0], (0, 0))

    # print(dir(text_image))

    while not done:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                done = True
            if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                done = True

        screen.fill((0, 0, 0))
        # screen.blit(text,
        screen.blit(
            text_image,
            (320 - text[0].get_width() // 2, 240 - text[0].get_height() // 2),
        )

        pygame.display.flip()
        clock.tick(60)
